<?php

// SPDX-FileCopyrightText: 2026 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Businessprocess\Controllers;

use Icinga\Module\Monitoring\Controller;
use Icinga\Web\Url;

class IdoHostController extends Controller
{
    public function showAction(): void
    {
        $hostName = $this->params->get('host');

        $query = $this->backend->select()
            ->from('hoststatus', array('host_name'))
            ->where('host_name', $hostName);

        $this->applyRestriction('monitoring/filter/objects', $query);
        if ($query->fetchRow() !== false) {
            $this->redirectNow(Url::fromPath('monitoring/host/show')->setParams($this->params));
        } else {
            $this->view->host = $hostName;
        }
    }
}
