<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Businessprocess\Web\Form\Element;

use Zend_Form_Element_Xhtml;

class FormElement extends Zend_Form_Element_Xhtml
{
}
