<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Businessprocess\Web\Component;

use ipl\Html\ValidHtml;

class Tabs extends WtfTabs implements ValidHtml
{
}
