<?php

// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Businessprocess\Web\Form\Element;

class Checkbox extends \Icinga\Web\Form\Element\Checkbox
{
}
